-- ElvUI Companions Datatext
-- By Lockslap, US - Bleeding Hollow
local E, L, V, P, G	= unpack(ElvUI)
local DT			= E:GetModule("DataTexts")
local LibQTip		= LibStub("LibQTip-1.0")

local join	= string.join
local split	= string.split
local band	= bit.band
local wipe	= table.wipe

local Tip, lastPanel
local displayString = ""
local colorString = "|cff%s%s|r"

local specialMounts = {
	[30174]		= true,	-- riding turtle
	[64731]		= true,	-- sea turtle
	[26054]		= true,	-- aq'40 bugs
	[25953]		= true,	-- ditto
	[26056]		= true,	-- same
	[26055]		= true,	-- mhm
}

local defaults = {
	["modifier"]		= "shift",
	["columns"]			= 3,
	
	["swimColor"]		= {r = 0,		g = 0.44,	b = 0.87},		-- rare (blue)
	["groundColor"]		= {r = 0.64,	g = 0.21,	b = 0.93},		-- epic (purple)
	["flyingColor"]		= {r = 1.0,		g = 0.5,	b = 0},			-- legendary (orange)
	["specialColor"]	= {r = 0.0,		g = 1.0,	b = 0},			-- uncommon (green)
	["petColor"]		= {r = 0.01,	g = 0.64,	b = 0.8},		-- turqoise-ish
	["selectColor"]		= {r = 1.0,		g = 0.0,	b = 0.56},		-- pink-ish
}

local function ToHex(color)
	return ("%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
end

local function GetMountColor(flags)
	local ground, flyer, swimmer = band(flags, 0x01) ~= 0, band(flags, 0x02) ~= 0, band(flags, 0x08) ~= 0
	
	if swimmer == true and ground ~= true and flyer ~= true then
		return mountdb.swimColor
	elseif ground == true and flyer ~= true then
		return mountdb.groundColor
	elseif flyer == true then
		return mountdb.flyingColor
	else
		return mountdb.specialColor
	end
end

local function IsModifierDown()
	if mountdb.modifier == "shift" then
		return IsShiftKeyDown() and true or false
	elseif mountdb.modifier == "ctrl" then
		return IsControlKeyDown() and true or false
	elseif mountdb.modifier == "alt" then
		return IsAltKeyDown() and true or false
	else
		return false
	end
end

local function CompanionType()
	return not IsModifierDown() and "MOUNT" or "CRITTER"
end

local function CompanionClick(frame, info, button)
	local id, active = split(":", info)
	if tonumber(active) == 1 then
		DismissCompanion(CompanionType())
	else
		CallCompanion(CompanionType(), tonumber(id))
	end
	Tip:Hide()
end

local function OnEnter(self)
	if InCombatLockdown() then return end
	local pets = {}
	local header = not IsModifierDown() and L["Mounts"] or L["Non-Combat Pets"]
	local companion = CompanionType()
	
	if LibQTip:IsAcquired("ElvUI_Companions") then
		Tip:Clear()
	else
		Tip = LibQTip:Acquire("ElvUI_Companions", mountdb.columns)
		Tip:SmartAnchorTo(self)
		Tip:SetAutoHideDelay(0.1, self)
		Tip:SetScript("OnShow", function(self) self:SetTemplate("Transparent") end)
	end
	
	line = Tip:AddHeader()
	Tip:SetCell(line, 1, ("|cffffffff%s|r"):format(header), "LEFT")
	Tip:AddSeparator()
	
	if GetNumCompanions(companion) == 0 then
		line = Tip:AddLine()
		Tip:SetCell(line, 1, ("|cffff0000No %s Found!|r"):format(header), "CENTER", 3)
	else
		wipe(pets)
		
		for i = 1, GetNumCompanions(companion) do
			local _, name, spell, _, active, flags = GetCompanionInfo(companion, i)
			local colored
			
			if companion == "CRITTER" then
				colored = colorString:format(ToHex(active == 1 and mountdb.selectColor or mountdb.petColor), name)
			else
				if active == 1 then
					colored = colorString:format(ToHex(mountdb.selectColor), name)
				elseif specialMounts[spell] ~= nil then
					colored = colorString:format(ToHex(mountdb.specialColor), name)
				else
					colored = colorString:format(ToHex(GetMountColor(flags)), name)
				end
			end
			
			pets[#pets + 1] = {
				id		= i,
				raw		= name,
				name	= colored,
				active	= active,
			}
		end
		
		-- no funny business
		if #pets == 0 then return end
		
		-- sort by the companions raw name, w/o any coloring
		sort(pets, function(a, b) return a["raw"] < b["raw"] end)
		
		local i = 1
		for index, value in pairs(pets) do
			if i == 1 then line = Tip:AddLine() end
			Tip:SetCell(line, i, value.name, "LEFT", nil, nil, nil, nil, nil, 200)
			Tip:SetCellScript(line, i, "OnMouseUp", CompanionClick, tostring(value.id) .. ":" .. tostring(value.active))
			
			if i == mountdb.columns then
				i = 1
			else
				i = i + 1
			end
		end
	end
	
	Tip:UpdateScrolling()
	Tip:Show()
end

local function OnEvent(self, event, ...)
	-- hack to make sure the settings are always up to date
	mountdb = mountdb or {}
	for index, value in pairs(defaults) do
		if type(mountdb[index]) == "nil" then
			mountdb[index] = value
		end
	end
	
	E.Options.args.datatexts.args.companions = {
		type		= "group",
		order		= 25,
		name		= L["Companions"],
		guiInline	= true,
		get			= function(info) return mountdb[info[#info]] end,
		set			= function(info, value) mountdb[info[#info]] = value; DT:LoadDataTexts() end,
		args		= {
			swimColor		= {
				type	= "color",
				order	= 1,
				name	= L["Swimming Mount Color"],
				desc	= L["Color to use for swimming mounts."],
				get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
				set		= function(info, r, g, b)
					mountdb[info[#info]].r = r
					mountdb[info[#info]].g = g
					mountdb[info[#info]].b = b
				end,
			},
			groundColor		= {
				type	= "color",
				order	= 2,
				name	= L["Ground Mount Color"],
				desc	= L["Color to use for ground mounts."],
				get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
				set		= function(info, r, g, b)
					mountdb[info[#info]].r = r
					mountdb[info[#info]].g = g
					mountdb[info[#info]].b = b
				end,
			},
			flyingColor		= {
				type	= "color",
				order	= 3,
				name	= L["Flying Mount Color"],
				desc	= L["Color to use for flying mounts."],
				get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
				set		= function(info, r, g, b)
					mountdb[info[#info]].r = r
					mountdb[info[#info]].g = g
					mountdb[info[#info]].b = b
				end,
			},
			specialColor	= {
				type	= "color",
				order	= 4,
				name	= L["Special Mount Color"],
				desc	= L["Color to use for the \"special\" mounts.  Like the bugs from AQ."],
				get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
				set		= function(info, r, g, b)
					mountdb[info[#info]].r = r
					mountdb[info[#info]].g = g
					mountdb[info[#info]].b = b
				end,
			},
			petColor		= {
				type	= "color",
				order	= 5,
				name	= L["Pet Color"],
				desc	= L["Color to use for pets."],
				get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
				set		= function(info, r, g, b)
					mountdb[info[#info]].r = r
					mountdb[info[#info]].g = g
					mountdb[info[#info]].b = b
				end,
			},
			selectColor		= {
				type	= "color",
				order	= 6,
				name	= L["Selected Color"],
				desc	= L["Color for selected companions."],
				get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
				set		= function(info, r, g, b)
					mountdb[info[#info]].r = r
					mountdb[info[#info]].g = g
					mountdb[info[#info]].b = b
				end,
			},
			modifier = {
				type	= "select",
				order	= 7,
				name	= L["Modifier"],
				desc	= L["Key to hold to swap between mounts and companions."],
				values	= {
					["alt"]		= L["Alt"],
					["ctrl"]	= L["Ctrl"],
					["shift"]	= L["Shift"],
				},
			},
			columns = {
				type	= "range",
				order	= 8,
				name	= L["Columns"],
				desc	= L["The number of columns in the tooltip."],
				min = 1, max = 5, step = 1,
				set		= function(info, value)
					mountdb[info[#info]] = value
					if LibQTip:IsAcquired("ElvUI_Companions") then
						LibQTip:Release(Tip)
					end
				end,
			},
		}
	}
	
	self.text:SetFormattedText(displayString,
		not IsModifierDown() and L["Mounts"] or L["Pets"],
		GetNumCompanions(not IsModifierDown() and "MOUNT" or "CRITTER"))
end

local function OnUpdate(self, elapsed)
	self.text:SetFormattedText(displayString,
		not IsModifierDown() and L["Mounts"] or L["Pets"],
		GetNumCompanions(not IsModifierDown() and "MOUNT" or "CRITTER"))
end

local function Click(self, button)
	if button == "LeftButton" then
		ToggleSpellBook("mount")
	elseif button == "RightButton" then
		ToggleSpellBook("companions")
	end
end

local function ValueColorUpdate(hex, r, g, b)
	displayString = join("", "|cffffffff%s:|r", " ", hex, "%d|r")
	if lastPanel ~= nil then OnEvent(lastPanel) end
end
E["valueColorUpdateFuncs"][ValueColorUpdate] = true

DT:RegisterDatatext("Companions", {"PLAYER_ENTERING_WORLD"}, OnEvent, OnUpdate, Click, OnEnter)